@@ -36,6 +36,8 @@ import openstreetmap
 import pango
 import threadpool
 logger = logging.getLogger('gtkmap')
+from os.path import getsize
+from hashlib import md5
 
 
 class Map(gtk.DrawingArea, AbstractMap):
@@ -80,7 +82,9 @@ class Map(gtk.DrawingArea, AbstractMap):
 
         self.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("white"))
 
-
+    def shutdown(self):
+        self.tile_loader_threadpool.dismissWorkers(openstreetmap.CONCURRENT_THREADS * 2)
+        self.tile_loader_threadpool.joinAllDismissedWorkers()
 
         ##############################################
         #
@@ -339,6 +343,11 @@ class Map(gtk.DrawingArea, AbstractMap):
         surface = cairo.ImageSurface.create_from_png(filename)
         if surface.get_width() != surface.get_height():
             raise Exception("Image too small, probably corrupted file")
+            
+        # Filter out OpenStreetMap illegal URL tile
+        if surface.get_format() == cairo.FORMAT_ARGB32: # Check format first, because it's very fast.
+            if getsize(filename) == 3713 and md5(surface.get_data()).hexdigest() == '6d9afab2f24e9660b7689b11bf11c880':
+                raise Exception("Illegal OSM tile.")
         return surface
 
     def __run_tile_loader(self, id_string, tile, zoom, undersample, x, y, callback_draw):
@@ -506,6 +515,8 @@ class GeocacheLayer(AbstractGeocacheLayer):
     COLOR_MULTI = gtk.gdk.color_parse('orange')
     COLOR_CACHE_CENTER = gtk.gdk.color_parse('black')
     COLOR_CURRENT_CACHE = gtk.gdk.color_parse('red')
+    COLOR_DISABLED = gtk.gdk.color_parse('red')
+    COLOR_ARCHIVED = gtk.gdk.color_parse('blue')
     COLOR_WAYPOINTS = gtk.gdk.color_parse('deeppink')
 
     def draw(self):
@@ -607,7 +618,16 @@ class GeocacheLayer(AbstractGeocacheLayer):
             # if this cache is disabled
             if c.status == geocaching.GeocacheCoordinate.STATUS_DISABLED:
                 cr.set_line_width(3)
-                cr.set_source_color(self.COLOR_CURRENT_CACHE)
+                cr.set_source_color(self.COLOR_DISABLED)
+                radius_disabled = 7
+                cr.move_to(p[0]-radius_disabled, p[1]-radius_disabled)
+                cr.line_to(p[0] + radius_disabled, p[1] + radius_disabled)
+                cr.stroke()
+
+            # if this cache is archived
+            if c.status == geocaching.GeocacheCoordinate.STATUS_ARCHIVED:
+                cr.set_line_width(3)
+                cr.set_source_color(self.COLOR_ARCHIVED)
                 radius_disabled = 7
                 cr.move_to(p[0]-radius_disabled, p[1]-radius_disabled)
                 cr.line_to(p[0] + radius_disabled, p[1] + radius_disabled)
